# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lekko/backend/v1beta1/distribution_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from lekko_client.gen.lekko.feature.v1beta1 import feature_pb2 as lekko_dot_feature_dot_v1beta1_dot_feature__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0lekko/backend/v1beta1/distribution_service.proto\x12\x15lekko.backend.v1beta1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a#lekko/feature/v1beta1/feature.proto\"K\n\rRepositoryKey\x12\x1d\n\nowner_name\x18\x01 \x01(\tR\townerName\x12\x1b\n\trepo_name\x18\x02 \x01(\tR\x08repoName\"\x7f\n\x1bGetRepositoryVersionRequest\x12?\n\x08repo_key\x18\x01 \x01(\x0b\x32$.lekko.backend.v1beta1.RepositoryKeyR\x07repoKey\x12\x1f\n\x0bsession_key\x18\x02 \x01(\tR\nsessionKey\"=\n\x1cGetRepositoryVersionResponse\x12\x1d\n\ncommit_sha\x18\x01 \x01(\tR\tcommitSha\"\xca\x01\n\x1cGetRepositoryContentsRequest\x12?\n\x08repo_key\x18\x01 \x01(\x0b\x32$.lekko.backend.v1beta1.RepositoryKeyR\x07repoKey\x12%\n\x0enamespace_name\x18\x02 \x01(\tR\rnamespaceName\x12!\n\x0c\x66\x65\x61ture_name\x18\x03 \x01(\tR\x0b\x66\x65\x61tureName\x12\x1f\n\x0bsession_key\x18\x04 \x01(\tR\nsessionKey\"\x80\x01\n\x1dGetRepositoryContentsResponse\x12\x1d\n\ncommit_sha\x18\x01 \x01(\tR\tcommitSha\x12@\n\nnamespaces\x18\x02 \x03(\x0b\x32 .lekko.backend.v1beta1.NamespaceR\nnamespaces\"[\n\tNamespace\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12:\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32\x1e.lekko.backend.v1beta1.FeatureR\x08\x66\x65\x61tures\"i\n\x07\x46\x65\x61ture\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x10\n\x03sha\x18\x02 \x01(\tR\x03sha\x12\x38\n\x07\x66\x65\x61ture\x18\x03 \x01(\x0b\x32\x1e.lekko.feature.v1beta1.FeatureR\x07\x66\x65\x61ture\"2\n\nContextKey\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x12\n\x04type\x18\x02 \x01(\tR\x04type\"\x8f\x03\n\x13\x46lagEvaluationEvent\x12?\n\x08repo_key\x18\x01 \x01(\x0b\x32$.lekko.backend.v1beta1.RepositoryKeyR\x07repoKey\x12\x1d\n\ncommit_sha\x18\x02 \x01(\tR\tcommitSha\x12\x1f\n\x0b\x66\x65\x61ture_sha\x18\x03 \x01(\tR\nfeatureSha\x12%\n\x0enamespace_name\x18\x04 \x01(\tR\rnamespaceName\x12!\n\x0c\x66\x65\x61ture_name\x18\x05 \x01(\tR\x0b\x66\x65\x61tureName\x12\x44\n\x0c\x63ontext_keys\x18\x06 \x03(\x0b\x32!.lekko.backend.v1beta1.ContextKeyR\x0b\x63ontextKeys\x12\x1f\n\x0bresult_path\x18\x07 \x03(\x05R\nresultPath\x12\x46\n\x11\x63lient_event_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0f\x63lientEventTime\"\x87\x01\n SendFlagEvaluationMetricsRequest\x12\x42\n\x06\x65vents\x18\x01 \x03(\x0b\x32*.lekko.backend.v1beta1.FlagEvaluationEventR\x06\x65vents\x12\x1f\n\x0bsession_key\x18\x02 \x01(\tR\nsessionKey\"#\n!SendFlagEvaluationMetricsResponse\"\xdc\x01\n\x15RegisterClientRequest\x12?\n\x08repo_key\x18\x01 \x01(\x0b\x32$.lekko.backend.v1beta1.RepositoryKeyR\x07repoKey\x12%\n\x0enamespace_list\x18\x02 \x03(\tR\rnamespaceList\x12\x32\n\x15initial_bootstrap_sha\x18\x03 \x01(\tR\x13initialBootstrapSha\x12\'\n\x0fsidecar_version\x18\x04 \x01(\tR\x0esidecarVersion\"9\n\x16RegisterClientResponse\x12\x1f\n\x0bsession_key\x18\x04 \x01(\tR\nsessionKey\":\n\x17\x44\x65registerClientRequest\x12\x1f\n\x0bsession_key\x18\x01 \x01(\tR\nsessionKey\"\x1a\n\x18\x44\x65registerClientResponse\" \n\x1eGetDeveloperAccessTokenRequest\"7\n\x1fGetDeveloperAccessTokenResponse\x12\x14\n\x05token\x18\x01 \x01(\tR\x05token2\xa8\x06\n\x13\x44istributionService\x12\x81\x01\n\x14GetRepositoryVersion\x12\x32.lekko.backend.v1beta1.GetRepositoryVersionRequest\x1a\x33.lekko.backend.v1beta1.GetRepositoryVersionResponse\"\x00\x12\x84\x01\n\x15GetRepositoryContents\x12\x33.lekko.backend.v1beta1.GetRepositoryContentsRequest\x1a\x34.lekko.backend.v1beta1.GetRepositoryContentsResponse\"\x00\x12\x90\x01\n\x19SendFlagEvaluationMetrics\x12\x37.lekko.backend.v1beta1.SendFlagEvaluationMetricsRequest\x1a\x38.lekko.backend.v1beta1.SendFlagEvaluationMetricsResponse\"\x00\x12o\n\x0eRegisterClient\x12,.lekko.backend.v1beta1.RegisterClientRequest\x1a-.lekko.backend.v1beta1.RegisterClientResponse\"\x00\x12u\n\x10\x44\x65registerClient\x12..lekko.backend.v1beta1.DeregisterClientRequest\x1a/.lekko.backend.v1beta1.DeregisterClientResponse\"\x00\x12\x8a\x01\n\x17GetDeveloperAccessToken\x12\x35.lekko.backend.v1beta1.GetDeveloperAccessTokenRequest\x1a\x36.lekko.backend.v1beta1.GetDeveloperAccessTokenResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'lekko.backend.v1beta1.distribution_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_REPOSITORYKEY']._serialized_start=145
  _globals['_REPOSITORYKEY']._serialized_end=220
  _globals['_GETREPOSITORYVERSIONREQUEST']._serialized_start=222
  _globals['_GETREPOSITORYVERSIONREQUEST']._serialized_end=349
  _globals['_GETREPOSITORYVERSIONRESPONSE']._serialized_start=351
  _globals['_GETREPOSITORYVERSIONRESPONSE']._serialized_end=412
  _globals['_GETREPOSITORYCONTENTSREQUEST']._serialized_start=415
  _globals['_GETREPOSITORYCONTENTSREQUEST']._serialized_end=617
  _globals['_GETREPOSITORYCONTENTSRESPONSE']._serialized_start=620
  _globals['_GETREPOSITORYCONTENTSRESPONSE']._serialized_end=748
  _globals['_NAMESPACE']._serialized_start=750
  _globals['_NAMESPACE']._serialized_end=841
  _globals['_FEATURE']._serialized_start=843
  _globals['_FEATURE']._serialized_end=948
  _globals['_CONTEXTKEY']._serialized_start=950
  _globals['_CONTEXTKEY']._serialized_end=1000
  _globals['_FLAGEVALUATIONEVENT']._serialized_start=1003
  _globals['_FLAGEVALUATIONEVENT']._serialized_end=1402
  _globals['_SENDFLAGEVALUATIONMETRICSREQUEST']._serialized_start=1405
  _globals['_SENDFLAGEVALUATIONMETRICSREQUEST']._serialized_end=1540
  _globals['_SENDFLAGEVALUATIONMETRICSRESPONSE']._serialized_start=1542
  _globals['_SENDFLAGEVALUATIONMETRICSRESPONSE']._serialized_end=1577
  _globals['_REGISTERCLIENTREQUEST']._serialized_start=1580
  _globals['_REGISTERCLIENTREQUEST']._serialized_end=1800
  _globals['_REGISTERCLIENTRESPONSE']._serialized_start=1802
  _globals['_REGISTERCLIENTRESPONSE']._serialized_end=1859
  _globals['_DEREGISTERCLIENTREQUEST']._serialized_start=1861
  _globals['_DEREGISTERCLIENTREQUEST']._serialized_end=1919
  _globals['_DEREGISTERCLIENTRESPONSE']._serialized_start=1921
  _globals['_DEREGISTERCLIENTRESPONSE']._serialized_end=1947
  _globals['_GETDEVELOPERACCESSTOKENREQUEST']._serialized_start=1949
  _globals['_GETDEVELOPERACCESSTOKENREQUEST']._serialized_end=1981
  _globals['_GETDEVELOPERACCESSTOKENRESPONSE']._serialized_start=1983
  _globals['_GETDEVELOPERACCESSTOKENRESPONSE']._serialized_end=2038
  _globals['_DISTRIBUTIONSERVICE']._serialized_start=2041
  _globals['_DISTRIBUTIONSERVICE']._serialized_end=2849
# @@protoc_insertion_point(module_scope)
