# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lekko/feature/v1beta1/static.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from lekko_client.gen.lekko.feature.v1beta1 import feature_pb2 as lekko_dot_feature_dot_v1beta1_dot_feature__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"lekko/feature/v1beta1/static.proto\x12\x15lekko.feature.v1beta1\x1a#lekko/feature/v1beta1/feature.proto\"\x9c\x02\n\rStaticFeature\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x36\n\x04type\x18\x02 \x01(\x0e\x32\".lekko.feature.v1beta1.FeatureTypeR\x04type\x12@\n\x07imports\x18\x03 \x03(\x0b\x32&.lekko.feature.v1beta1.ImportStatementR\x07imports\x12>\n\x07\x66\x65\x61ture\x18\x04 \x01(\x0b\x32$.lekko.feature.v1beta1.FeatureStructR\x07\x66\x65\x61ture\x12?\n\x0b\x66\x65\x61ture_old\x18\x05 \x01(\x0b\x32\x1e.lekko.feature.v1beta1.FeatureR\nfeatureOld\"\xd5\x01\n\rFeatureStruct\x12\x33\n\x04meta\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarMetaR\x04meta\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription\x12\x39\n\x07\x64\x65\x66\x61ult\x18\x03 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarExprR\x07\x64\x65\x66\x61ult\x12\x32\n\x05rules\x18\x04 \x01(\x0b\x32\x1c.lekko.feature.v1beta1.RulesR\x05rules\"o\n\x05Rules\x12\x33\n\x04meta\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarMetaR\x04meta\x12\x31\n\x05rules\x18\x02 \x03(\x0b\x32\x1b.lekko.feature.v1beta1.RuleR\x05rules\"\x90\x01\n\x04Rule\x12\x33\n\x04meta\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarMetaR\x04meta\x12\x1c\n\tcondition\x18\x02 \x01(\tR\tcondition\x12\x35\n\x05value\x18\x03 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarExprR\x05value\"\xea\x01\n\x0fImportStatement\x12\x33\n\x04meta\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarMetaR\x04meta\x12\x32\n\x03lhs\x18\x02 \x01(\x0b\x32 .lekko.feature.v1beta1.IdentExprR\x03lhs\x12\x1a\n\x08operator\x18\x03 \x01(\tR\x08operator\x12\x1d\n\nline_break\x18\x04 \x01(\x08R\tlineBreak\x12\x33\n\x03rhs\x18\x05 \x01(\x0b\x32!.lekko.feature.v1beta1.ImportExprR\x03rhs\"\x87\x01\n\nImportExpr\x12\x33\n\x04meta\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarMetaR\x04meta\x12\x30\n\x03\x64ot\x18\x02 \x01(\x0b\x32\x1e.lekko.feature.v1beta1.DotExprR\x03\x64ot\x12\x12\n\x04\x61rgs\x18\x03 \x03(\tR\x04\x61rgs\"`\n\x07\x44otExpr\x12\x33\n\x04meta\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarMetaR\x04meta\x12\x0c\n\x01x\x18\x02 \x01(\tR\x01x\x12\x12\n\x04name\x18\x03 \x01(\tR\x04name\"_\n\x08StarExpr\x12\x33\n\x04meta\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarMetaR\x04meta\x12\x1e\n\nexpression\x18\x02 \x01(\tR\nexpression\"V\n\tIdentExpr\x12\x33\n\x04meta\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.StarMetaR\x04meta\x12\x14\n\x05token\x18\x02 \x01(\tR\x05token\"e\n\x08StarMeta\x12;\n\x08\x63omments\x18\x01 \x01(\x0b\x32\x1f.lekko.feature.v1beta1.CommentsR\x08\x63omments\x12\x1c\n\tmultiline\x18\x02 \x01(\x08R\tmultiline\"\xb0\x01\n\x08\x43omments\x12\x36\n\x06\x62\x65\x66ore\x18\x01 \x03(\x0b\x32\x1e.lekko.feature.v1beta1.CommentR\x06\x62\x65\x66ore\x12\x36\n\x06suffix\x18\x02 \x03(\x0b\x32\x1e.lekko.feature.v1beta1.CommentR\x06suffix\x12\x34\n\x05\x61\x66ter\x18\x03 \x03(\x0b\x32\x1e.lekko.feature.v1beta1.CommentR\x05\x61\x66ter\"\x1f\n\x07\x43omment\x12\x14\n\x05token\x18\x01 \x01(\tR\x05tokenb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'lekko.feature.v1beta1.static_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_STATICFEATURE']._serialized_start=99
  _globals['_STATICFEATURE']._serialized_end=383
  _globals['_FEATURESTRUCT']._serialized_start=386
  _globals['_FEATURESTRUCT']._serialized_end=599
  _globals['_RULES']._serialized_start=601
  _globals['_RULES']._serialized_end=712
  _globals['_RULE']._serialized_start=715
  _globals['_RULE']._serialized_end=859
  _globals['_IMPORTSTATEMENT']._serialized_start=862
  _globals['_IMPORTSTATEMENT']._serialized_end=1096
  _globals['_IMPORTEXPR']._serialized_start=1099
  _globals['_IMPORTEXPR']._serialized_end=1234
  _globals['_DOTEXPR']._serialized_start=1236
  _globals['_DOTEXPR']._serialized_end=1332
  _globals['_STAREXPR']._serialized_start=1334
  _globals['_STAREXPR']._serialized_end=1429
  _globals['_IDENTEXPR']._serialized_start=1431
  _globals['_IDENTEXPR']._serialized_end=1517
  _globals['_STARMETA']._serialized_start=1519
  _globals['_STARMETA']._serialized_end=1620
  _globals['_COMMENTS']._serialized_start=1623
  _globals['_COMMENTS']._serialized_end=1799
  _globals['_COMMENT']._serialized_start=1801
  _globals['_COMMENT']._serialized_end=1832
# @@protoc_insertion_point(module_scope)
