# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lekko/feature/v1beta1/feature.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from lekko_client.gen.lekko.rules.v1beta2 import rules_pb2 as lekko_dot_rules_dot_v1beta2_dot_rules__pb2
from lekko_client.gen.lekko.rules.v1beta3 import rules_pb2 as lekko_dot_rules_dot_v1beta3_dot_rules__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#lekko/feature/v1beta1/feature.proto\x12\x15lekko.feature.v1beta1\x1a\x19google/protobuf/any.proto\x1a\x1flekko/rules/v1beta2/rules.proto\x1a\x1flekko/rules/v1beta3/rules.proto\"\xa6\x01\n\x07\x46\x65\x61ture\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription\x12/\n\x04tree\x18\x03 \x01(\x0b\x32\x1b.lekko.feature.v1beta1.TreeR\x04tree\x12\x36\n\x04type\x18\x04 \x01(\x0e\x32\".lekko.feature.v1beta1.FeatureTypeR\x04type\"\xb8\x01\n\x04Tree\x12.\n\x07\x64\x65\x66\x61ult\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyR\x07\x64\x65\x66\x61ult\x12\x43\n\x0b\x63onstraints\x18\x02 \x03(\x0b\x32!.lekko.feature.v1beta1.ConstraintR\x0b\x63onstraints\x12;\n\x0b\x64\x65\x66\x61ult_new\x18\x03 \x01(\x0b\x32\x1a.lekko.feature.v1beta1.AnyR\ndefaultNew\"\xc1\x02\n\nConstraint\x12\x12\n\x04rule\x18\x01 \x01(\tR\x04rule\x12*\n\x05value\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyR\x05value\x12\x43\n\x0b\x63onstraints\x18\x03 \x03(\x0b\x32!.lekko.feature.v1beta1.ConstraintR\x0b\x63onstraints\x12\x38\n\x08rule_ast\x18\x04 \x01(\x0b\x32\x19.lekko.rules.v1beta2.RuleB\x02\x18\x01R\x07ruleAst\x12;\n\x0crule_ast_new\x18\x05 \x01(\x0b\x32\x19.lekko.rules.v1beta3.RuleR\nruleAstNew\x12\x37\n\tvalue_new\x18\x06 \x01(\x0b\x32\x1a.lekko.feature.v1beta1.AnyR\x08valueNew\"6\n\x03\x41ny\x12\x19\n\x08type_url\x18\x01 \x01(\tR\x07typeUrl\x12\x14\n\x05value\x18\x02 \x01(\x0cR\x05value*\xb8\x01\n\x0b\x46\x65\x61tureType\x12\x1c\n\x18\x46\x45\x41TURE_TYPE_UNSPECIFIED\x10\x00\x12\x15\n\x11\x46\x45\x41TURE_TYPE_BOOL\x10\x01\x12\x14\n\x10\x46\x45\x41TURE_TYPE_INT\x10\x02\x12\x16\n\x12\x46\x45\x41TURE_TYPE_FLOAT\x10\x03\x12\x17\n\x13\x46\x45\x41TURE_TYPE_STRING\x10\x04\x12\x15\n\x11\x46\x45\x41TURE_TYPE_JSON\x10\x05\x12\x16\n\x12\x46\x45\x41TURE_TYPE_PROTO\x10\x06\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'lekko.feature.v1beta1.feature_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _CONSTRAINT.fields_by_name['rule_ast']._options = None
  _CONSTRAINT.fields_by_name['rule_ast']._serialized_options = b'\030\001'
  _globals['_FEATURETYPE']._serialized_start=892
  _globals['_FEATURETYPE']._serialized_end=1076
  _globals['_FEATURE']._serialized_start=156
  _globals['_FEATURE']._serialized_end=322
  _globals['_TREE']._serialized_start=325
  _globals['_TREE']._serialized_end=509
  _globals['_CONSTRAINT']._serialized_start=512
  _globals['_CONSTRAINT']._serialized_end=833
  _globals['_ANY']._serialized_start=835
  _globals['_ANY']._serialized_end=889
# @@protoc_insertion_point(module_scope)
